package model;

import java.util.LinkedList;
import java.util.List;

public class Alarm implements Comparable<Alarm>, Operable, ObservableSubject {
	private class AlarmState implements Cloneable {
	    public String name;
	    public Time time;
	    public boolean enabled;
	    public AlarmState() {
	    	name = null;
	    	time = null;
	    	enabled = true;
	    }
	    public AlarmState(String n, String t)
	    throws BadHourException, BadMinuteException, BadFormatException {
	    	name = n;
	    	time = new Time(t);
	    	enabled = true;
	    }
	    public Object clone() {
	    	try {
	    		return (AlarmState)super.clone();
	    	}
	    	catch (CloneNotSupportedException e) {
	    		return null;
	    	}
	    }
	}
	private AlarmState state;
    private List<Observer> observers = new LinkedList<Observer>();
    public Alarm() {
    	state = new AlarmState();
    }
    public Alarm(String name, String time)
    throws BadHourException, BadMinuteException, BadFormatException {
        state = new AlarmState(name, time);
    }
    public String getName() {
        return state.name;
	}
	public void setName(String name) {
		state.name = name;
	}
	public Time getTime() {
		return state.time;
	}
	public void setTime(Time t) {
		state.time = t;
	}
	public void setTime(String time)
    throws BadHourException, BadMinuteException, BadFormatException {
		state.time = new Time(time);
	}
	public void setEnabled() {
		state.enabled = true;
	}
	public boolean isEnabled() {
		return state.enabled;
	}
	public void setDisabled() {
		state.enabled = false;
	}
	public String toString() {
		if (state.enabled) return state.time + " : " + state.name;
		return state.time + " : ( " + state.name + " )";
	}
	public int compareTo(Alarm o) {
		int resultForTimes = state.time.compareTo(o.state.time);
		if (resultForTimes == 0)
			return state.name.compareTo(o.state.name);
		return resultForTimes;
	}
	public void performs(Operation op) throws Exception {
		op.doItOn(this);
	}
	public void addObserver(Observer o) {
		observers.add(o);
		System.out.println("add : " + observers.size());
	}
	public void removeObserver(Observer o) {
		observers.remove(o);
		System.out.println("remove : " + observers.size());
	}
	public void notifyObservers() {
		System.out.println("notify : " + observers.size());
		for(Observer o : observers) o.update();
	}
	public Memento createMemento() {
		Memento m = new Memento(this, state.clone());
		return m;
	}
	public void restoreFrom(Memento m) {
		AlarmState s = (AlarmState)m.getState();
		state.name = s.name;
		state.time = s.time;
		state.enabled = s.enabled;
		notifyObservers();
	}
}
